import logging

from django.core.exceptions import ValidationError
from django.db import IntegrityError

from .models import BpStructure
from .utils import (
    StructureCache,
    StructureFillField,
    get_gaby_struct,
    get_skip_status,
)

logger = logging.getLogger(__name__)


def run():
    """
    import records from biopic and merge them
    into gaby
    """
    logger.info('attempt to merge biopic into gaby')
    # initialize a cache of slugify(gaby.structure.designation)
    gaby_str_cache = StructureCache()
    gaby_str_fill = StructureFillField()
    cpt = 0
    cpt_existant = 0
    cpt_import = 0
    cpt_import_err = 0
    for bpstruct in BpStructure.objects.not_npdc():
        skip_struct, reason = get_skip_status(bpstruct)
        if skip_struct:
            logger.info(
                "{}({}) ignoré ({})".format(bpstruct, bpstruct.id, reason)
            )
            continue
        cpt += 1
        struct, reason = get_gaby_struct(bpstruct, gaby_str_cache)
        if struct is not None:
            cpt_existant += 1
            logger.info(
                "{}({}) trouvé dans gaby({}), "
                "ajout d'un lien vers {}({})".format(
                    bpstruct, bpstruct.id, reason, struct, struct.id
                )
            )
            struct.url_old_ref = bpstruct.url()
            struct.save()
            gaby_str_fill.add_tag_import_manuel(struct)
            continue
        # at this point it's a brand new structure to import from biopic
        cpt_import += 1
        struct = gaby_str_fill.build_from(bpstruct)
        struct_is_clean = True
        err_validation = None
        try:
            struct.full_clean()
        except ValidationError as err:
            cpt_import_err += 1
            err_validation = err
            struct_is_clean = False
        try:
            struct.save()
        except IntegrityError as err:
            logger.error(
                "{}({}) erreur critique d'importation: [{}]".format(
                    bpstruct, bpstruct.id, err
                )
            )
            raise  # raise anyway, the transaction is broken
        else:
            if err_validation is not None:
                for f_name, f_errors in err_validation.message_dict.items():
                    logger.warning(
                        "{} erreur de validation: {}[{}] ({})".format(
                            struct.get_absolute_url(),
                            f_name,
                            " / ".join(f_errors),
                            struct,
                        )
                    )
            # log it
            logger.debug(
                "{}({}) importé dans gaby: {}".format(
                    bpstruct, bpstruct.id, struct.get_absolute_url()
                )
            )
            # a new structure has been successfully imported into gaby
            # override create_date with date_maj
            gaby_str_fill.reset_create_date(bpstruct, struct)
            struct.save()
            # add m2m fields
            gaby_str_fill.add_m2m(bpstruct, struct, struct_is_clean)
            # add related fields
            gaby_str_fill.add_related(bpstruct, struct)
            # update the search cache
            gaby_str_cache.add_to_cache(struct)
            # at this point related fields (M2M and FK to this struct)
            # could be imported
            # import personne physique / contact
            # import adhesion
            # import misc.
    logger.info(
        "Stats: Total: {}, Existant: {}, Importé(total): {}, "
        "Importé(avec erreur de validation): {}".format(
            cpt, cpt_existant, cpt_import, cpt_import_err
        )
    )
