import datetime
import logging
from decimal import Decimal

from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.utils.text import slugify

from gaby.contacts.fields_choices import CHOIX_VISIBILITE
from gaby.contacts.models import (
    ActiviteStructure,
    AdhesionGabnor,
    Assolement,
    Commercialisation,
    DonneesBio,
    Personne,
    ProjetBio,
    Structure,
)
from gaby.references.models import (
    Commune,
    Culture,
    DescCategorieStructure,
    DescTypeVenteStructure,
    Etiquette,
    FormeJuridique,
    StatutStructure,
)

from . import tables
from .models import BpStructure, Categorie, TypeVente

logger = logging.getLogger(__name__)


class SlugCache:
    model = None
    slug_fields = []

    def __init__(self):
        self._slug_dict = {}
        for obj in self.model.objects.all():
            self.add_to_cache(obj)

    def add_to_cache(self, obj):
        slug_name = "-".join(
            [
                slugify(getattr(obj, slug_field))
                for slug_field in self.slug_fields
                if getattr(obj, slug_field, False)
            ]
        )
        if slug_name:
            id_list = self._slug_dict.get(slug_name, None)
            if id_list is None:
                self._slug_dict[slug_name] = []
            self._slug_dict[slug_name].append(obj)

    def search(self, *names):
        slug_name = "-".join([slugify(name) for name in names if name])
        return self._slug_dict.get(slug_name, None)

    def get_dupes(self):
        return [(k, v) for k, v in self._slug_dict.items() if len(v) > 1]


class StructureCache(SlugCache):
    model = Structure
    slug_fields = ['designation']


class BpStructureCache(SlugCache):
    model = BpStructure
    slug_fields = ['nom']


class CommuneCache(SlugCache):
    model = Commune
    slug_fields = ['nom']


class PersonneCache(SlugCache):
    model = Personne
    slug_fields = ['prenom', 'nom']


def get_skip_status(bpstruct):
    if bpstruct.id in tables.STR_BLACKLIST:
        return True, 'tableur_biohdf'
    if (
        bpstruct.cp
        and len(bpstruct.cp) == 5
        and bpstruct.cp[:2] in ('59', '62')
    ):
        return True, 'cp_npdc'
    if bpstruct.statut_bio == 'fin activite':
        return True, 'fin_activite'
    return False, None


def get_gaby_struct(bpstruct, gaby_str_cache):
    """
    match sur la liste envoyée par Bertrand
    puis sur le même numéro de siret
    puis sur un nom de structure similaire
    renvoi le candidat et la raison
    """
    struct_id = tables.STR_ABP2GABY.get(bpstruct.id, None)
    if struct_id is not None:
        return Structure.objects.get(id=struct_id), 'tableur_biohdf'
    if bpstruct.siret:
        siret_candidate = Structure.objects.filter(
            siret=bpstruct.siret.strip()
        ).first()
        if siret_candidate is not None:
            return siret_candidate, 'siret_identique'
    candidates = gaby_str_cache.search(bpstruct.nom)
    if candidates is not None:
        # search() cannot return empty list by design
        return (candidates[0], 'nom_similaire')
    return None, None


class StructureFillField:
    def __init__(self):
        # initialise le(s) cache(s)
        self._f_juridique = {
            key: FormeJuridique.objects.get_or_create(designation=val)
            for key, val in tables.MTX_F_JUR.items()
        }
        self._statut = {
            key: StatutStructure.objects.get_or_create(designation=val)
            for key, val in tables.MTX_STATUT_STR.items()
        }
        self._etiquette = {
            key: Etiquette.objects.get_or_create(designation=val)
            for key, val in tables.MTX_ETIQUETTE.items()
        }
        self._desc_categorie = {
            key: DescCategorieStructure.objects.get_or_create(designation=key)
            for key in Categorie.objects.values_list('nom', flat=True)
        }
        self._desc_type_vente = {
            key: DescTypeVenteStructure.objects.get_or_create(designation=key)
            for key in TypeVente.objects.values_list('nom', flat=True)
        }
        self._culture = {
            key: Culture.objects.get_or_create(designation=val)
            for key, val in tables.MTX_CULTURE.items()
        }
        self.admin_user = User.objects.first()
        self.communes_cache = CommuneCache()

    def build_from(self, bpstruct):
        struct = Structure(**self.map_from_biopic(bpstruct))
        return struct

    def map_from_biopic(self, bpstruct):
        res = {
            'url_old_ref': bpstruct.url(),
            'utilisateur_derniere_modification': self.admin_user,
            'statutbio': tables.MTX_STATUTBIO.get(bpstruct.statut_bio, 0),
            'statut': self.get_statut(bpstruct),
            'f_juridique': self._f_juridique.get(bpstruct.statut)[0],
            'designation': bpstruct.nom[:250],
            'siret': bpstruct.siret.strip()[:14] if bpstruct.siret else '',
            'adresse': bpstruct.adresse or '',
            'site_web': self.get_site_web(bpstruct),
            'tel_fixe': bpstruct.tel[:100] if bpstruct.tel else '',
            'tel_mobile': bpstruct.tel2[:100] if bpstruct.tel2 else '',
            'geolocalisation': ",".join(
                [e for e in [bpstruct.latitude, bpstruct.longitude] if e]
            ),
            'courriel': ','.join(
                [e for e in [bpstruct.mail, bpstruct.mail2] if e]
            )[:255],
            'commentaire': bpstruct.observation or '',
            'code_postal': bpstruct.cp or '',
            'commune': self.get_commune(bpstruct),
            'visibilite': CHOIX_VISIBILITE[bool(bpstruct.diffusion_web)][0],
            'description_activite': bpstruct.description_activite or '',
            'description_activite_map': bpstruct.description_activite_map
            or '',
            'description_web': bpstruct.description_web or '',
        }
        res.update(self.get_ville_cedex(bpstruct, res['commune']))
        res.update(self.get_cat_acteur(bpstruct))
        return res

    def add_tag_import_manuel(self, struct):
        "ajout etiquette 'import_abp_manuel' "
        struct.etiquettes.add(self._etiquette['import_abp_manuel'][0])

    def add_m2m(self, bpstruct, struct, struct_is_clean):
        etiquettes = [self._etiquette['import_abp'][0]]
        if not struct_is_clean:
            etiquettes.append(self._etiquette['import_abp_err'][0])
        struct.etiquettes.set(etiquettes)
        categories = [
            self._desc_categorie[elm][0]
            for elm in bpstruct.categories.values_list('nom', flat=True)
        ]
        struct.description_categorie.set(categories)
        type_ventes = [
            self._desc_type_vente[elm][0]
            for elm in bpstruct.type_ventes.values_list('nom', flat=True)
        ]
        struct.description_typevente.set(type_ventes)

    def get_cat_acteur(self, bpstruct):
        res = {}
        list_act_autre = []
        if bpstruct.cat_activite is None:
            return res
        for act in bpstruct.cat_activite.split(','):
            attr = tables.MTX_CAT_ACT.get(act, None)
            if attr is None:
                list_act_autre.append(act)
            else:
                res[attr] = True
        if list_act_autre:
            res['cat_act_autre'] = ','.join(list_act_autre)
        return res

    def get_commune(self, bpstruct):
        "cherche une commune correspondante via insee_id ou dans le cache"
        com = None
        if bpstruct.insee_id:
            com = Commune.objects.filter(code_insee=bpstruct.insee_id).first()
        if com is None and bpstruct.ville:
            candidates = self.communes_cache.search(bpstruct.ville)
            if candidates:
                com = candidates[0]
        return com

    def get_ville_cedex(self, bpstruct, commune):
        "remplit ville ssi cedex existe"
        res = {}
        if bpstruct.cedex:
            res['ville'] = " ".join(
                [
                    e
                    for e in [
                        commune.nom if commune else bpstruct.ville,
                        bpstruct.cedex,
                    ]
                    if e
                ]
            )
        return res

    def get_statut(self, bpstruct):
        res = self._statut.get(bpstruct.statut, [None, None])[0]
        if bpstruct.porteur_projet is not None:
            if res is None:
                res = self._statut.get('_projet')[0]
            else:
                logger.warning(
                    "{}({}) a un statut incohérent: "
                    "projet et fin d'activité".format(bpstruct, bpstruct.id)
                )
        return res

    def get_site_web(self, bpstruct):
        if not bpstruct.web:
            return None
        site_web = bpstruct.web
        if not site_web.startswith('http'):
            site_web = 'http://' + site_web
        return site_web[:250]

    def reset_create_date(self, bpstruct, struct):
        "si maj_date existe on remet a zero date_creation"
        if bpstruct.date_maj and isinstance(bpstruct.date_maj, datetime.date):
            struct.date_creation = bpstruct.date_maj

    def add_related(self, bpstruct, struct):
        self.add_related_persons(bpstruct, struct)
        self.add_related_activites(bpstruct, struct)
        self.add_related_projetbio(bpstruct, struct)
        self.add_related_adhesion(bpstruct, struct)
        self.add_related_donneesbio(bpstruct, struct)

    def add_related_persons(self, bpstruct, struct):
        "struct doit deja avoir un id"
        # on termine par structure.*_ref
        for contact in bpstruct.contact_set.all():
            pers = Personne(
                utilisateur_derniere_modification=self.admin_user,
                structure=struct,
                nom=contact.nom[:255],
                prenom=(contact.prenom or '')[:255],
                civilite=tables.MTX_CIVILITE.get(contact.titre, ''),
                fonction=(contact.fonction or '')[:255],
                courriel=(contact.mail or '')[:255],
                tel_fixe=(contact.telephone or '')[:100],
                tel_mobile=(contact.mobile or '')[:100],
                fax=(contact.fax or '')[:100],
                adresse=contact.adresse or '',
                code_postal=contact.cp or '',
                ville=(contact.ville or '')[:255],
                pas_de_courriel=(
                    contact.preference_contact is not None
                    and 'mail' not in contact.preference_contact
                ),
                date_naissance=(
                    datetime.date(contact.annee_naissance, 1, 1)
                    if contact.annee_naissance
                    else None
                ),
                date_installation=(
                    datetime.date(contact.annee_installation, 1, 1)
                    if contact.annee_installation
                    else None
                ),
            )
            try:
                pers.full_clean()
            except ValidationError as err:
                for field_name, field_errors in err.message_dict.items():
                    logger.warning(
                        "{} erreur de validation: {}[{}] ({})".format(
                            struct.get_absolute_url(),
                            field_name,
                            " / ".join(field_errors),
                            contact,
                        )
                    )
            try:
                pers.save()
            except IntegrityError as err:
                logger.error(
                    "{}({}) erreur critique d'importation: [{}]".format(
                        contact, contact.id, err
                    )
                )
                raise  # raise anyway, transaction is broken
            # at this point there is a new personne
            logger.debug(
                "attache {}({}) a la structure {}({})".format(
                    contact, contact.id, struct, struct.id
                )
            )

    def add_related_activites(self, bpstruct, struct):
        # on determine si on est conventionnel, projet ou bio
        activite_bio = 2 if struct.statutbio == 2 else 0
        if (
            activite_bio == 2
            and struct.statut
            and struct.statut.designation == 'PROJET'
        ):
            activite_bio = 1
        # on ajout l'activite principale 'producteur_principal'
        if bpstruct.producteur_principal is None:
            return
        act_obj = ActiviteStructure.objects.create(
            structure=struct,
            activite_bio=activite_bio,
            nom=tables.MTX_ACT[bpstruct.producteur_principal],
        )
        logger.debug(
            "attache activité {} a la structure {}({})".format(
                act_obj, struct, struct.id
            )
        )
        # puis on enumere les activites  'producteur'
        if bpstruct.producteur is None:
            return
        for act in bpstruct.producteur.split(','):
            act_obj = ActiviteStructure.objects.create(
                structure=struct,
                activite_bio=activite_bio,
                nom=tables.MTX_ACT[act],
            )
            logger.debug(
                "attache activité {} a la structure {}({})".format(
                    act_obj, struct, struct.id
                )
            )

    def add_related_projetbio(self, bpstruct, struct):
        if not (bpstruct.passage_bio and bpstruct.passage_bio > 0):
            return
        projet_bio = ProjetBio.objects.create(
            agriculteur=struct,
            date_engagement_oc=datetime.date(bpstruct.passage_bio, 1, 1),
        )
        logger.debug(
            "attache Projet/date_engagement {} a la structure {}({})".format(
                projet_bio.date_engagement_oc, struct, struct.id
            )
        )

    def add_related_adhesion(self, bpstruct, struct):
        adhesions = bpstruct.adhesion_set.all()
        if not adhesions:
            return
        adhesions_gaby = [
            AdhesionGabnor(
                agriculteur_bio=struct,
                annee=adhesion.annee,
                date=datetime.date(adhesion.annee, 1, 1),
                montant=115,
            )
            for adhesion in adhesions
        ]
        # bulk create .save() is not called
        AdhesionGabnor.objects.bulk_create(adhesions_gaby)
        # manually call .save() to update related structure
        adhesions_gaby[0].save()
        for adh_gaby in adhesions_gaby:
            logger.debug(
                "attache adhesion {} a la structure {}({})".format(
                    adh_gaby, struct, struct.id
                )
            )

    def add_related_donneesbio(self, bpstruct, struct):
        """
        on commence a ajouter les valeurs dans orab_activites
        on extrait la derniere, sinon on la cree
        et on y ajoute les donnees ETP
        """
        annees = set()
        donneesbio = None
        for orab in bpstruct.orabactivite_set.order_by('annee'):
            if orab.annee in annees:
                continue
            annees.add(orab.annee)
            donneesbio = DonneesBio(
                agriculteur_bio=struct,
                annee=orab.annee,
                remarques="import ABP",
            )
            try:
                donneesbio.full_clean()
            except ValidationError as err:
                for field_name, field_errors in err.message_dict.items():
                    logger.warning(
                        "{} erreur de validation: {}[{}] ({})".format(
                            struct.get_absolute_url(),
                            field_name,
                            " / ".join(field_errors),
                            donneesbio,
                        )
                    )
            logger.debug(
                "attache DonneesBio {} a la structure {}({})".format(
                    donneesbio, struct, struct.id
                )
            )
            donneesbio.save()
            if orab.sau_total or orab.sau_bio or orab.sau_conversion:
                sau_bio = (orab.sau_bio or 0) + (orab.sau_conversion or 0)
                sau_total = max((orab.sau_total or 0), sau_bio)
                sau_conv = sau_total - sau_bio
                Assolement.objects.create(
                    donneesbio=donneesbio,
                    culture=self._culture['autre'][0],
                    surface_bio=sau_bio,
                    surface_conv=sau_conv,
                )
            if orab.ca_total or orab.ca_bio:
                ca_bio = int(orab.ca_bio or 0)
                ca_total = max(int(orab.ca_total or 0), ca_bio)
                ca_conv = ca_total - ca_bio
                if ca_conv:
                    Commercialisation.objects.create(
                        donneesbio=donneesbio, ch_aff=ca_conv, bio=False
                    )
                if ca_bio:
                    Commercialisation.objects.create(
                        donneesbio=donneesbio, ch_aff=ca_bio, bio=True
                    )
        # fin du for orab in ...
        if not (
            bpstruct.etp_total or bpstruct.etp_bio or bpstruct.etp_sal_app
        ):
            return
        if donneesbio is None:
            donneesbio = DonneesBio(
                agriculteur_bio=struct,
                annee=bpstruct.date_maj.year if bpstruct.date_maj else 2019,
                remarques="import ABP",
            )
        logger.debug(
            "attache DonneesBio(ETP) {} a la structure {}({})".format(
                donneesbio, struct, struct.id
            )
        )
        if bpstruct.etp_total:
            donneesbio.etp_associes = round(Decimal(bpstruct.etp_total), 2)
        else:
            donneesbio.etp_associes = bpstruct.etp_bio and round(
                Decimal(bpstruct.etp_bio), 2
            )
            donneesbio.etp_salaries_permanents = (
                bpstruct.etp_sal_app and round(Decimal(bpstruct.etp_sal_app))
            )
        try:
            donneesbio.full_clean()
        except ValidationError as err:
            for field_name, field_errors in err.message_dict.items():
                logger.warning(
                    "{} erreur de validation: {}[{}] (ETP:{})".format(
                        struct.get_absolute_url(),
                        field_name,
                        " / ".join(field_errors),
                        donneesbio,
                    )
                )
        donneesbio.save()
