# -*- coding: utf-8 -*-
import re

# from django.conf import settings
# from django.contrib import admin
import django.forms as forms

import gaby.contacts.models as contacts_models

# from contacts.logger import logger


# ## Données et fonctions utilitaires

# Définition du widget utilisé pour les différentes adresse
# On le centralise ici pour qu'il soit partout pareil.
_widget_adresse = {'adresse': forms.Textarea(attrs={'rows': 3, 'cols': 40})}


# ## Form d'admin de modèles
class ContactAdminForm(forms.ModelForm):
    """
    Personnalisation du formulaire pour limiter l'adresse sur 4 lignes (pour
    simplifier la génération d'étiquettes) et affecter automatiquement les
    Cette classe n'est pas utilisée directement, mais est utilisable par
    héritage pour tous les modèles représentant un contact.

    La sous-classe Meta doit être définie par les classes filles pour
    désigner le modèle qui s'applique. Meta n'est pas hérité.

    class Meta:
       model = ...
       widgets = _widget_adresse.copy()
    """

    def clean_adresse(self):
        """ Limite l'adresse à 4 lignes. """
        addr = self.cleaned_data['adresse']
        if len(re.findall('\n', addr)) > 3:
            raise forms.ValidationError(
                "L'adresse ne doit pas comporter plus de 4 lignes."
            )
        return addr


class StructureAdminForm(ContactAdminForm):
    """
    Personnalisation du formulaire pour limiter l'adresse sur 4 lignes (pour
    simplifier la génération d'étiquettes).
    """

    class Meta:
        model = contacts_models.Structure
        widgets = _widget_adresse.copy()
        exclude = []


class PersonneAdminForm(ContactAdminForm):
    """ Personnalisation du formulaire pour limiter l'adresse sur 4
    lignes (pour simplifier la génération d'étiquettes)."""

    class Meta:
        model = contacts_models.Personne
        widgets = _widget_adresse.copy()
        exclude = []


class RdvForm(forms.ModelForm):
    class Meta:
        model = contacts_models.Rdv
        exclude = []

    def __init__(self, *args, **kwargs):
        super(RdvForm, self).__init__(*args, **kwargs)
        self._pm_pk = self._get_pm(kwargs.get('initial', {}))
        self._filter_projetbio()

    def _get_pm(self, initial):
        "lookup associated personne morale"
        pm_pk = (
            self.instance
            and self.instance.personne_morale
            and self.instance.personne_morale.pk
        )
        pm_pk = pm_pk or initial.get('personne_morale')
        return pm_pk

    def _filter_projetbio(self):
        """restreint la liste des projets bio affichés en fonction de
        la personne morale si celle ci est indiquée, sinon affiche une
        liste vide"""
        fd_projet_bio = self.fields.get('projet_bio')
        # qs inchangée en cas de POST, si besoin validation, cf clean()
        if fd_projet_bio and not self.is_bound:
            qs_projet_bio = getattr(fd_projet_bio, 'queryset', None)
            pm_pk = self._pm_pk
            fd_projet_bio.queryset = (
                qs_projet_bio.filter(agriculteur=pm_pk)
                if pm_pk
                else qs_projet_bio.none()
            )


# EOF
