from django.contrib.auth import models as am
from django.test import TestCase
from django.urls import reverse

from gaby.contacts import models as cm
from gaby.contacts.tests.utils import cfty
from gaby.references import models as rm


class ValDonneesBioView(TestCase):
    def setUp(self):
        self.usr1 = am.User.objects.create_superuser('usr1', '', 'mdp')
        self.usr1.save()
        self.usr2 = am.User.objects.create_user('usr2', '', 'mdp2')
        self.usr2.save()
        self.fjur = rm.FormeJuridique()
        self.fjur.save()
        self.culture = rm.Culture()
        self.culture.save()
        self.i_pm = {
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
            'f_juridique': self.fjur,
            'designation': "test",
            'statutbio': 1,
            'adherent': cm.AdhLogin.objects.get(username='usr2'),
        }
        self.structure = cfty(cm.Structure, self.i_pm)
        self.data = {
            'agriculteur_bio': self.structure.id,
            'annee': 2013,
            'assolement_set-TOTAL_FORMS': '0',
            'assolement_set-INITIAL_FORMS': '0',
            'assolement_set-MAX_NUM_FORMS': '',
            'ass_total': '0',
            'ass_total_bio': '0',
            'commercialisation_set-TOTAL_FORMS': '0',
            'commercialisation_set-INITIAL_FORMS': '0',
            'commercialisation_set-MAX_NUM_FORMS': '',
            'transformation_set-TOTAL_FORMS': '0',
            'transformation_set-INITIAL_FORMS': '0',
            'transformation_set-MAX_NUM_FORMS': '',
        }

    def test_creation(self):
        """La premiere fois que des donnees bio sont entrees il ne doit pas
        y avoir de commercialisation/assolement/transformation
        Lorsque des donnees ont deja ete entrees les annees precedentes il
        faut reprendre ces informations
        Lorsque des donnees ont deja ete entrees pour l'annee en cours
        il faut reconduire vers ces donnees"""

        # En passant par la vue admin d'une structure

        self.client.login(username=self.usr1.username, password='mdp')
        add_path = reverse('admin:cree_donneesbio', args=[self.structure.id])

        # Premiere creation de donnees bio
        response = self.client.get(add_path, follow=True)
        self.check_inline(response, 0, 0, 0)
        path = response.request['PATH_INFO']
        self.assertEqual(cm.DonneesBio.objects.count(), 1)

        # Insertion de donnees
        data = self.data.copy()
        data['lait_nb_vaches'] = '100'
        data['lait_vaches'] = 'True'
        data['assolement_set-TOTAL_FORMS'] = '1'
        data['assolement_set-0-culture'] = str(self.culture.id)
        response = self.client.post(path, data)
        response = self.client.get(path)
        self.assertEqual(
            response.context['adminform'].form.initial['lait_nb_vaches'], 100
        )
        self.check_inline(response, 1, 0, 0)

        # Creation d'une autre fiche de donnees qui doit comporter les donnees
        # precedemment entrees
        response = self.client.get(add_path, follow=True)
        self.assertNotEqual(response.request['PATH_INFO'], path)
        self.assertEqual(
            response.context['adminform'].form.initial['lait_nb_vaches'], 100
        )
        self.check_inline(response, 1, 0, 0)

        # Creation ou non des donnees si on ne clique pas sur enregistrer
        # Mettre a 1 si il n'y a pas de creation
        self.assertEqual(cm.DonneesBio.objects.count(), 2)
        self.assertEqual(cm.Assolement.objects.count(), 2)

    def test_creation_client(self):
        """La premiere fois que des donnees bio sont entrees il ne doit pas
        y avoir de commercialisation/assolement/transformation
        Lorsque des donnees ont deja ete entrees les annees precedentes il
        faut reprendre ces informations
        Lorsque des donnees ont deja ete entrees pour l'annee en cours
        il faut reconduire vers ces donnees"""

        self.client.login(username=self.usr2.username, password='mdp2')

        # Premiere creation de donnees bio
        add_path = '/adh/' + str(self.structure.id) + '/donneebio/add/'
        response = self.client.get(add_path)
        self.assertEqual(response.status_code, 200)
        self.check_inline_client(response, 0, 0, 0)

        # Insertion de donnees
        data = self.data.copy()
        data['lait_nb_vaches'] = '100'
        data['lait_vaches'] = 'True'
        data['assolement_set-TOTAL_FORMS'] = '1'
        data['assolement_set-0-culture'] = str(self.culture.id)
        response = self.client.post(add_path, data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(cm.DonneesBio.objects.count(), 1)
        first_path = response.request['PATH_INFO']
        donneesbio = cm.DonneesBio.objects.order_by('-annee')[0]
        donneesbio.annee = 2012
        donneesbio.save()
        response = self.client.get(first_path + 'edit/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context['form'].initial['lait_nb_vaches'], 100
        )
        self.check_inline_client(response, 1, 0, 0)

        # Creation d'une autre fiche de donnees qui doit comporter les donnees
        # precedemment entrees
        # Avant enregistrement
        response = self.client.get(add_path)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context['form'].initial['lait_nb_vaches'], 100
        )
        self.check_inline_client(response, 1, 0, 0)
        # Apres enregistrement
        response = self.client.post(add_path, data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(cm.DonneesBio.objects.count(), 2)
        second_path = response.request['PATH_INFO']
        donneesbio = cm.DonneesBio.objects.order_by('-annee')[0]
        donneesbio.annee = 2013
        donneesbio.save()
        response = self.client.get(second_path + 'edit/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context['form'].initial['lait_nb_vaches'], 100
        )
        self.check_inline_client(response, 1, 0, 0)

        # Creation d'une autre fiche de donnees qui doit comporter les donnees
        # precedemment entrees en plus des nouvelles donnees
        response = self.client.get(add_path)
        self.assertEqual(response.status_code, 200)
        data = self.data.copy()
        data['commercialisation_set-TOTAL_FORMS'] = '1'
        data['assolement_set-TOTAL_FORMS'] = '1'
        data['commercialisation_set-0-type_produit'] = 'autre'
        data['commercialisation_set-0-type_circuit'] = 'autre'
        data['commercialisation_set-0-bio'] = True
        data['lait_nb_vaches'] = response.context['form'][
            'lait_nb_vaches'
        ].value()
        data['lait_vaches'] = response.context['form']['lait_vaches'].value()
        data['assolement_set-0-culture'] = response.context[
            'assolement_formset'
        ][0].initial['culture']
        response = self.client.post(add_path, data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(cm.DonneesBio.objects.count(), 3)
        third_path = response.request['PATH_INFO']
        donneesbio = cm.DonneesBio.objects.order_by('-annee')[0]
        donneesbio.annee = 2014
        donneesbio.save()
        response = self.client.get(third_path + 'edit/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context['form'].initial['lait_nb_vaches'], 100
        )
        self.check_inline_client(response, 1, 0, 1)

        # Si la duplication doit creer des donnees meme sans cliquer sur
        # enregistrer mettre la valeur a 4
        response = self.client.get(add_path, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(cm.DonneesBio.objects.count(), 3)

        # Verification de la redirection lorsque les donnees de l'annee en cours
        # existent deja
        response1 = self.client.post(add_path, data, follow=True)
        response2 = self.client.get(add_path, follow=True)
        self.assertEqual(
            response1.request['PATH_INFO'], response2.request['PATH_INFO']
        )

    def check_inline(self, response, nb_assol, nb_trans, nb_commerc):
        nb_assolement = response.context['inline_admin_formsets'][
            0
        ].formset.total_form_count()
        nb_transformation = response.context['inline_admin_formsets'][
            1
        ].formset.total_form_count()
        nb_commercialisation = response.context['inline_admin_formsets'][
            2
        ].formset.total_form_count()
        self.assertEqual(nb_assolement, nb_assol)
        self.assertEqual(nb_transformation, nb_trans)
        self.assertEqual(nb_commercialisation, nb_commerc)

    def check_inline_client(self, response, nb_assol, nb_trans, nb_commerc):
        nb_assolement = response.context[
            'assolement_formset'
        ].total_form_count()
        nb_transformation = response.context[
            'transformation_formset'
        ].total_form_count()
        nb_commercialisation = response.context[
            'commercialisation_formset'
        ].total_form_count()
        self.assertEqual(nb_assolement, nb_assol)
        self.assertEqual(nb_transformation, nb_trans)
        self.assertEqual(nb_commercialisation, nb_commerc)
