# -*- coding: utf-8 -*-
import datetime

import django.views.generic as gviews
from django import forms
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.db import models
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse

import gaby.contacts.forms as cforms
import gaby.contacts.model_forms as cmforms
import gaby.contacts.models as cmodels
from gaby.contacts.cl_views import ListFilter
from gaby.contacts.logger import logger

MTX_RAND = {
    None: lambda x: '' if x is None else x,
    #    models.DateField: lambda x: '' if x is None else x,
    #    models.CharField: lambda x: '' if x is None else x,
    models.BooleanField: lambda x: 'oui' if x else 'non',
    models.NullBooleanField: lambda x: 'inconnu'
    if x is None
    else ('oui' if x else 'non'),
}


def small_render(model, field, value):
    """
    render field value
    """
    if hasattr(field, 'choices') and field.choices:
        res_val = getattr(model, 'get_{}_display'.format(field.name))()
        return "" if res_val is None else res_val
    return MTX_RAND.get(field.__class__, MTX_RAND[None])(value)


def test_view(request, code=None):
    "permet de tester les cas d'erreur"
    if code == "404":
        raise Http404("Testé 404")
    if code == "403":
        raise PermissionDenied("Testé 403")
    if code == "500":
        return  # should raise an error 500
    if code == "warning":
        logger.warning("testé: warning")
    return render(request, 'test.html', {'title': code})


def gen_stat():
    "recupere des infos generales sur la Bdd"
    return {
        'nb_total': cmodels.Structure.objects.count(),
        'nb_agri': cmodels.Structure.objects.filter(statutbio__gte=1).count(),
        'nb_agri_bio': cmodels.Structure.objects.filter(statutbio=2).count(),
    }


class AccueilPublic(gviews.TemplateView):
    template_name = "accueil_public.html"

    def get_context_data(self, *args, **kwargs):
        context = super(AccueilPublic, self).get_context_data(*args, **kwargs)
        context.update({'stat': gen_stat(), 'title': "Accueil"})
        return context


class PubStructureListFilter(ListFilter):
    form_class = cforms.StructureFilterForm
    template_name = "contacts/structure_list.html"
    model = cmodels.Structure
    extra_context = {'title': "Annuaire"}

    def get_queryset(self):
        "Les agriculteur qui ont choisi d'apparaitre"
        b_filter = models.Q(visibilite='tous')
        if self.request.adh:
            b_filter |= models.Q(visibilite='adherents')
        return self.model.objects.filter(b_filter, statutbio__gte=2)


class PubStructureDetail(gviews.DetailView):
    model = cmodels.Structure
    template_name = 'contacts/structure_detail.html'
    extra_context = {'title': "Détail de la structure"}

    def get_context_data(self, **kwargs):
        """
        Insert the single object into the context dict.
        """
        context = {}
        if self.object:
            context['object'] = self.object
            try:
                context['donneesbio'] = cmodels.DonneesBio.objects.filter(
                    agriculteur_bio__id=self.object.id
                ).order_by('-annee')[0]
            except IndexError:
                context['donneesbio'] = None
            context_object_name = self.get_context_object_name(self.object)
            if context_object_name:
                context[context_object_name] = self.object
        context.update(kwargs)
        return super(PubStructureDetail, self).get_context_data(**context)

    def get_queryset(self):
        "Les agriculteur qui ont choisi d'apparaitre"
        type_visi = ['tous']
        kw_filter = {'visibilite__in': type_visi, 'statutbio__gte': 2}
        if self.request.user.is_anonymous:
            qs_filter = models.Q(**kw_filter)
        else:
            type_visi.append('adherents')
            qs_adh_struct = self.request.adh.structure_set.all()
            qs_filter = models.Q(**kw_filter) | models.Q(id__in=qs_adh_struct)
        return self.model.objects.filter(qs_filter)


class AdhObjMixin(object):
    """
    Mixin pour selectionner les objets de l'adherent
    """

    def get_queryset(self):
        if self.model is None:
            return []
        if self.request.user.is_anonymous:
            return self.model.objects.none()
        return self.model.objects.filter(adherent_id=self.request.user.id)

    def get_object(self):
        if self.request.user.is_anonymous:
            return None
        obj_l = self.model.objects.filter(adherent_id=self.request.user.id)
        return obj_l[0] if len(obj_l) > 0 else None


class DetailAdh(AdhObjMixin, gviews.ListView):
    """
    Affichage des structures affiliee a un utilisateur
    """

    template_name = "contacts/adh_profile.html"
    model = cmodels.Structure


class UpdateStructAdh(AdhObjMixin, gviews.UpdateView):
    """
    Mise a jour des informations de contacts via un objet "shadow"
    """

    model = cmodels.Structure


class DetailDonneesBioAdh(gviews.DetailView):
    """
    Affichage de mes donnees bio
    """

    model = cmodels.DonneesBio
    template_name = "contacts/donneesbio_detail.html"

    def get_queryset(self):
        return self.model.objects.filter(
            agriculteur_bio__adherent=self.request.user
        )

    def get_context_data(self, **kwargs):
        object = self.get_object()
        assolements = cmodels.Assolement.objects.filter(
            donneesbio__pk=object.pk
        )
        commercialisations = cmodels.Commercialisation.objects.filter(
            donneesbio__pk=object.pk
        )
        transformations = cmodels.Transformation.objects.filter(
            donneesbio__pk=object.pk
        )
        sup = super(DetailDonneesBioAdh, self)
        exclude = cmforms.DonneesBioAdhForm._meta.exclude
        rof = [
            (
                elm.verbose_name,
                small_render(self.object, elm, getattr(self.object, elm.name)),
            )
            for elm in self.object._meta.fields
            if elm.name not in exclude
        ]
        exclude = 'donneesbio'
        rofAssol = []
        for assol in assolements:
            rofAssol += [
                (
                    elm.verbose_name,
                    small_render(assol, elm, getattr(assol, elm.name)),
                )
                for elm in assol._meta.fields
                if elm.name not in exclude
            ]
        rofComm = []
        for comm in commercialisations:
            rofComm += [
                (
                    elm.verbose_name,
                    small_render(comm, elm, getattr(comm, elm.name)),
                )
                for elm in comm._meta.fields
                if elm.name not in exclude
            ]
        rofTrans = []
        for trans in transformations:
            rofTrans += [
                (
                    elm.verbose_name,
                    small_render(trans, elm, getattr(trans, elm.name)),
                )
                for elm in trans._meta.fields
                if elm.name not in exclude
            ]
        kwargs.update(
            {
                'rof': rof,
                'rofAssol': rofAssol,
                'rofComm': rofComm,
                'rofTrans': rofTrans,
            }
        )
        return sup.get_context_data(**kwargs)


class UpdateDonneesBioAdh(gviews.UpdateView):
    """
    mise a jour des donnees bio par l'adherent
    """

    model = cmodels.DonneesBio
    form_class = cmforms.DonneesBioAdhForm
    template_name = "contacts/donneesbio_form.html"

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.form = self.get_form(self.form_class)

        self.assolement_formset = cmforms.AssolementFormSet(
            self.request.POST, instance=self.object
        )
        self.commercialisation_formset = cmforms.CommercialisationFormSet(
            self.request.POST, instance=self.object
        )
        self.transformation_formset = cmforms.TransformationFormSet(
            self.request.POST, instance=self.object
        )
        if (
            self.form.is_valid()
            and self.assolement_formset.is_valid()
            and self.commercialisation_formset.is_valid()
            and self.transformation_formset.is_valid()
        ):
            return self.form_valid(
                self.form,
                self.assolement_formset,
                self.commercialisation_formset,
                self.transformation_formset,
            )
        else:
            return self.form_invalid(
                self.form,
                self.assolement_formset,
                self.commercialisation_formset,
                self.transformation_formset,
            )

    def form_valid(
        self,
        form,
        assolement_formset,
        commercialisation_formset,
        transformation_formset,
    ):
        self.object = form.save()
        self.assolement_formset.save()
        self.commercialisation_formset.save()
        self.transformation_formset.save()
        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(
        self,
        form,
        assolement_formset,
        commercialisation_formset,
        transformation_formset,
    ):
        error = []
        if not assolement_formset.is_valid():
            for err in assolement_formset.errors:
                error += err
        if not commercialisation_formset.is_valid():
            for err in commercialisation_formset.errors:
                error += err
        if not transformation_formset.is_valid():
            for err in transformation_formset.errors:
                error += err
        return self.render_to_response(
            {
                'form': form,
                'assolement_formset': assolement_formset,
                'commercialisation_formset': commercialisation_formset,
                'transformation_formset': transformation_formset,
                'error': error,
            }
        )

    def get_context_data(self, *args, **kwargs):
        form_class = self.form_class
        self.form = self.get_form(form_class)
        self.object = self.get_object()
        self.assolement_formset = cmforms.AssolementFormSet(
            instance=self.object
        )
        self.commercialisation_formset = cmforms.CommercialisationFormSet(
            instance=self.object
        )
        self.transformation_formset = cmforms.TransformationFormSet(
            instance=self.object
        )
        data = super(UpdateDonneesBioAdh, self).get_context_data(
            *args, **kwargs
        )
        data.update(
            {
                'form': self.form,
                'assolement_formset': self.assolement_formset,
                'commercialisation_formset': self.commercialisation_formset,
                'transformation_formset': self.transformation_formset,
            }
        )
        return data

    def get_queryset(self):
        return self.model.objects.filter(
            date_validation__isnull=True,
            agriculteur_bio__adherent=self.request.user,
        )

    def get_success_url(self):
        return reverse('adh_donneesbio_detail', args=[self.get_object().pk])


class CreateDonneesBioAdh(gviews.CreateView):
    """
    remplissage des donnees bio par l'agriculteur
    """

    model = cmodels.DonneesBio
    form_class = cmforms.DonneesBioAddAdhForm

    def get_struct(self, pk):
        "ajoute la personne morale ciblee a la classe"
        if pk is None:
            return None
        try:
            struct = get_object_or_404(cmodels.Structure, pk=pk)
        except Http404:
            struct = None
        return struct

    def dispatch(self, *args, **kwargs):
        self.struct = self.get_struct(kwargs.pop('struct_pk', None))
        return super(CreateDonneesBioAdh, self).dispatch(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        try:
            self.db_last = self.model.objects.filter(
                agriculteur_bio=self.struct
            ).order_by('-annee', '-date_collecte')[0]
        except IndexError:
            self.db_last = None
        this_year = datetime.date.today().year

        if self.db_last and self.db_last.annee == this_year:
            messages.warning(
                request,
                "Les données de l'année {0} existent".format(this_year),
            )
            return redirect('adh_donneesbio_detail', self.db_last.pk)
        elif self.db_last:
            self.assolement_formset = cmforms.AssolementFormSet(
                instance=self.db_last
            )
            self.commercialisation_formset = cmforms.CommercialisationFormSet(
                instance=self.db_last
            )
            self.transformation_formset = cmforms.TransformationFormSet(
                instance=self.db_last
            )
        else:
            self.assolement_formset = cmforms.AssolementFormSet()
            self.commercialisation_formset = cmforms.CommercialisationFormSet()
            self.transformation_formset = cmforms.TransformationFormSet()
        return super(CreateDonneesBioAdh, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        form_class = self.form_class
        form = self.get_form(form_class)
        try:
            self.db_last = self.model.objects.filter(
                agriculteur_bio=self.struct
            ).order_by('-annee', '-date_collecte')[0]
        except IndexError:
            self.db_last = None
        if form.is_valid():
            agri = cmodels.Structure.objects.get(pk=self.struct.id)
            self.donneesbio = cmodels.DonneesBio(agriculteur_bio=agri)
            # Pour eviter l'erreur "QueryDict instance is immutable" et pouvoir
            # utiliser save_as_new=True on fait une copie de request.POST qui ne
            # sera pas immuable
            post = request.POST.copy()
            form = form_class(post or None, instance=self.donneesbio)
            self.donneesbio = form.save(commit=False)
            self.assolement_formset = cmforms.AssolementFormSet(
                post, instance=self.donneesbio, save_as_new=True
            )
            self.commercialisation_formset = cmforms.CommercialisationFormSet(
                post, instance=self.donneesbio, save_as_new=True
            )
            self.transformation_formset = cmforms.TransformationFormSet(
                post, instance=self.donneesbio, save_as_new=True
            )
            if (
                self.assolement_formset.is_valid()
                and self.commercialisation_formset.is_valid()
                and self.transformation_formset.is_valid()
            ):
                self.object = self.donneesbio
                return self.form_valid(
                    request,
                    self.donneesbio,
                    self.assolement_formset,
                    self.commercialisation_formset,
                    self.transformation_formset,
                )
            else:
                return self.form_invalid(
                    form,
                    self.assolement_formset,
                    self.commercialisation_formset,
                    self.transformation_formset,
                )
        else:
            self.assolement_formset = cmforms.AssolementFormSet(
                self.request.POST, instance=self.db_last
            )
            self.commercialisation_formset = cmforms.CommercialisationFormSet(
                self.request.POST, instance=self.db_last
            )
            self.transformation_formset = cmforms.TransformationFormSet(
                self.request.POST, instance=self.db_last
            )
            return self.form_invalid(
                form,
                self.assolement_formset,
                self.commercialisation_formset,
                self.transformation_formset,
            )

    def form_valid(
        self,
        request,
        donneesbio,
        assolement_formset,
        commercialisation_formset,
        transformation_formset,
    ):
        donneesbio.save()
        assolement_formset.save()
        commercialisation_formset.save()
        transformation_formset.save()

        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(
        self,
        form,
        assolement_formset,
        commercialisation_formset,
        transformation_formset,
    ):
        error = []
        if not assolement_formset.is_valid():
            for err in assolement_formset.errors:
                error += err
        if not commercialisation_formset.is_valid():
            for err in commercialisation_formset.errors:
                error += err
        if not transformation_formset.is_valid():
            for err in transformation_formset.errors:
                error += err
        return self.render_to_response(
            {
                'form': form,
                'assolement_formset': assolement_formset,
                'commercialisation_formset': commercialisation_formset,
                'transformation_formset': transformation_formset,
                'error': error,
            }
        )

    def get_form_kwargs(self, *args, **kwargs):
        ikw = super(CreateDonneesBioAdh, self).get_form_kwargs(*args, **kwargs)
        ikw.update({'struct': self.struct})
        return ikw

    def get_success_url(self):
        return reverse('adh_donneesbio_detail', args=[self.object.id])

    def get_initial(self):
        # get latest DonneesBio
        return (
            forms.model_to_dict(self.db_last)
            if getattr(self, 'db_last', None)
            else None
        )

    def get_context_data(self, *args, **kwargs):
        data = super(CreateDonneesBioAdh, self).get_context_data(
            *args, **kwargs
        )
        data.update(
            {
                'struct': self.struct,
                'db_last': self.db_last,
                'assolement_formset': self.assolement_formset,
                'commercialisation_formset': self.commercialisation_formset,
                'transformation_formset': self.transformation_formset,
            }
        )
        return data


class ListAdhValidationRecap(gviews.ListView):
    """
    liste des donnee en attente de validation
    """

    model = cmodels.DonneesBio
    template_name = "contacts/donneesbio_recap.html"
    paginate_by = 100

    def get_queryset(self):
        return self.model.objects.filter(
            date_validation__isnull=True, date_modification__isnull=False
        )


# EOF
