# -*- coding: utf-8 -*-
import re

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import F

from gaby.utils import regroup_choices

from .fields_choices import AGENCES_EAU, CHOIX_DEPARTEMENTS


def regroup_culture_cat():
    "regroup 'culture' by 'categorie_cul'"
    qs_elm = Culture.objects.filter(enabled=True).order_by(
        "categorie_cul__designation", "indice_ordre", "designation"
    )

    def get_elm_cat(elm):
        return elm.categorie_cul.designation if elm.categorie_cul else ''

    def elm_disp(elm):
        return elm.designation

    return regroup_choices(qs_elm, get_elm_cat, elm_disp=elm_disp)


# Données ------------------------------------------------------------------

# XAA a XAAAA avec variante A-xaaaaa
RE_VALID_CP = re.compile(r'^([A-Z]-)?[0-9][A-Z0-9]{2,4}$')
# XXXX ou XXXXX avec le cas de la corse (2A 2B)
RE_VALID_CP_FR = re.compile(r'^(2[AB]|[0-9]{1,2})[0-9]{3}$')


# Données géographiques -------------------------------------------------


class PetiteRegionAgricole(models.Model):
    """
    Région agricole.
    """

    class Meta:
        verbose_name = "petite région agricole"
        verbose_name_plural = "petites régions agricoles"
        ordering = ('nom',)

    code = models.IntegerField(unique=True)
    nom = models.CharField(max_length=255)
    grande_region = models.IntegerField(verbose_name="grande région agricole")
    departement = models.CharField(
        max_length=3, choices=CHOIX_DEPARTEMENTS, verbose_name="département"
    )

    def __str__(self):
        return "%s (%s)" % (self.nom, self.code)


class Arrondissement(models.Model):
    """
    Subdivision des départements.
    """

    class Meta:
        unique_together = ('numero', 'departement')
        ordering = ('nom',)

    nom = models.CharField(max_length=255)
    numero = models.IntegerField()
    departement = models.CharField(
        max_length=3, choices=CHOIX_DEPARTEMENTS, verbose_name="département"
    )
    region = models.IntegerField()

    def __str__(self):
        return self.nom


class Territoire(models.Model):
    """
    """

    class Meta:
        ordering = ('nom',)

    nom = models.CharField(max_length=255)

    def __str__(self):
        return self.nom


class InterCommunalite(models.Model):
    """
    """

    class Meta:
        ordering = ('nom',)

    nom = models.CharField(max_length=255)

    def __str__(self):
        return self.nom


class AutreTerritoire(models.Model):
    """
    """

    class Meta:
        ordering = ('nom',)

    nom = models.CharField(max_length=255)

    def __str__(self):
        return self.nom


class CommuneQuerySet(models.QuerySet):
    def departement(self):
        "ajoute le departement"
        return self.annotate(departement=F('code_insee') / 1000)


class Commune(models.Model):
    """
    Commune reliée à un canton, un arrondissement, une petite région agricole,
    et une zone.
    """

    class Meta:
        ordering = ('nom',)

    objects = CommuneQuerySet.as_manager()
    nom = models.CharField(max_length=255)
    code_insee = models.PositiveIntegerField(
        "num. INSEE", null=True, blank=True
    )
    code_postal = models.CharField(max_length=15, blank=True)
    arrondissement = models.ForeignKey(
        "Arrondissement", on_delete=models.CASCADE
    )
    territoire = models.ForeignKey("Territoire", on_delete=models.CASCADE)
    intercommunalite = models.ForeignKey(
        "InterCommunalite", blank=True, null=True, on_delete=models.CASCADE
    )
    autre_territoire = models.ForeignKey(
        "AutreTerritoire", blank=True, null=True, on_delete=models.CASCADE
    )
    petite_region = models.ForeignKey(
        "PetiteRegionAgricole", on_delete=models.CASCADE
    )
    enjeu_eau = models.NullBooleanField()
    agence_eau = models.CharField(
        blank=True, choices=AGENCES_EAU, max_length=16
    )

    def __str__(self):
        return "%s (%s)" % (self.nom, self.code_postal)

    def _clean_code_postal(self):
        "nettoie puis verifie que le CP est conforme"
        # nettoie le champ
        self.code_postal = self.code_postal.replace(' ', '').upper()
        if self.code_postal == '':
            return
        # si le champs est non vide verifie que le cp est valide
        if self.code_insee is None:
            # c'est un code postal, pas forcement francais
            match_valid_cp = RE_VALID_CP.match(self.code_postal)
            if not match_valid_cp:
                raise ValidationError(
                    {
                        'code_postal': "Un code postal doit être de la forme"
                        " P-XXXXX ou XXXXX"
                    }
                )
        else:  # si code_insee est renseigné, c'est un CP français
            match_valid_cp_fr = RE_VALID_CP_FR.match(self.code_postal)
            if not match_valid_cp_fr:
                raise ValidationError(
                    {
                        'code_postal': "Un code postal français doit être de"
                        " la forme XXXXX"
                    }
                )
            # si code_postal match, on le stock avec un '0' de padding a gauche
            self.code_postal = "{0:>05}".format(self.code_postal)

    def clean(self):
        self._clean_code_postal()

    def code_et_ville(self):
        """
        Retourne le code postal et la ville sous forme de chaîne de caractères
        (utilisé pour la création de l'adresse).
        """
        return "%s %s" % (self.code_postal, self.nom.upper())


# Autres données de référence -----------------------------------------------


class DesignationObjet(models.Model):
    """
    Modèle abstrait pour les listes d'objets utilisant un objet désignation.
    """

    class Meta:
        abstract = True
        ordering = ('designation',)

    designation = models.CharField(max_length=250, verbose_name="désignation")

    def __str__(self):
        return self.designation


class NomObjet(models.Model):
    """
    Modèle abstrait pour les listes d'objets utilisant un objet nom.
    """

    class Meta:
        abstract = True
        ordering = ('nom',)

    nom = models.CharField(max_length=250, verbose_name="nom")

    def __str__(self):
        return self.nom


class StatutStructure(DesignationObjet):
    """
    Liste des statuts possibles pour une structure:

    Source:
        Association
        EARL
        GAEC
        Individuelle
        SCEA
    """

    class Meta:
        verbose_name = "statut de structure"
        verbose_name_plural = "statuts de structure"
        ordering = ('designation',)


class FormeJuridique(DesignationObjet):
    "liste des forme juridique"

    class Meta:
        verbose_name = "Forme Juridique"
        verbose_name_plural = "Formes Juridiques"
        ordering = ('designation',)


class Laiterie(NomObjet):
    """
    Liste des laiteries.

    Source:
        Biolait
        Catillon
        Danone
        From de Th
        GIE Av Lait
        Laitnaa
        Milcobel
        Nestlé
        Novandie
        Prospé Ferm
        Sodiaal
        Ucanel
        VD
        VPM
    """

    class Meta:
        ordering = ('nom',)


class OrganismeDeCertification(NomObjet):
    """
    Liste des organismes de certification biologique.

    Source:
        Ecocert
        Qualité France
    """

    class Meta:
        verbose_name = "organisme de certification"
        verbose_name_plural = "organismes de certification"
        ordering = ('nom',)


class Etiquette(DesignationObjet):
    ""

    class Meta:
        verbose_name = "étiquette"
        verbose_name_plural = "étiquettes"
        ordering = ('designation',)

    description = models.CharField(
        max_length=255, blank=True, verbose_name="description", default=''
    )


class StatutCommercialisation(DesignationObjet):
    "Liste des statuts possibles pour les commercialisation en vente directe."

    class Meta:
        verbose_name = "statut commerc. en vente directe"
        verbose_name_plural = "statuts commerc. en vente directe"
        ordering = ('designation',)


class TypeService(DesignationObjet):
    "Liste des types de services offerts par les agriculteurs bio."

    class Meta:
        verbose_name = "type de service"
        verbose_name_plural = "types de service"
        ordering = ('designation',)


class CategorieCulture(DesignationObjet):
    "categorie de culture"

    class Meta:
        verbose_name = "Catégorie de cultures"
        verbose_name_plural = "Catégories de cultures"


class Culture(DesignationObjet):
    "Liste des cultures (utilisé dans les assolement de DonneesBio)."

    class Meta:
        verbose_name = "Culture"
        verbose_name_plural = "Cultures"
        ordering = ('categorie_cul__designation', 'indice_ordre')

    categorie_cul = models.ForeignKey(
        "CategorieCulture",
        verbose_name="Catégorie",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )
    indice_ordre = models.IntegerField(
        "Indice d'ordonnancement",
        null=True,
        blank=True,
        help_text="Permet d'ordonner les cultures dans leur catégorie",
    )
    enabled = models.BooleanField("actif", default=True)

    def __str__(self):
        return "{0.categorie_cul}/{0.designation}".format(self)


class ClientCulture(DesignationObjet):
    "Liste de clients pour les cultures."

    class Meta:
        verbose_name = "Client de culture"
        verbose_name_plural = "Clients de culture"
        ordering = ('designation',)


class Abattoir(NomObjet):
    "Abattoirs."

    class Meta:
        ordering = ('nom',)


class AcheteurOeufs(DesignationObjet):
    "Liste d'acheteur d'oeuf pour les cultures."

    class Meta:
        verbose_name = "Acheteur d'œufs"
        verbose_name_plural = "Acheteurs d'œufs"
        ordering = ('designation',)


class Formation(DesignationObjet):
    "Liste des formations suivies par les agriculteurs."

    class Meta:
        verbose_name = "Formation"
        verbose_name_plural = "Formations"
        ordering = ('designation',)


class TypeFormation(DesignationObjet):
    "Liste des types de formation que les agriculteurs peuvent vouloir suivre."

    class Meta:
        verbose_name = "Désignation Évenement"
        verbose_name_plural = "Désignations Évenements"
        ordering = ('designation',)


class FinanceProg(DesignationObjet):
    "Liste des financement/programme concerné"

    class Meta:
        verbose_name = "Financement/Programme"
        verbose_name_plural = "Financements/Programmes"
        ordering = ('designation',)


class PublicCible(NomObjet):
    "public cible"

    class Meta:
        verbose_name = "Public Cible"
        verbose_name_plural = "Publics Cible"
        ordering = ('nom',)


class DescCategorieStructure(DesignationObjet):
    class Meta:
        verbose_name = "Catégorie"
        ordering = ('designation',)


class DescTypeVenteStructure(DesignationObjet):
    class Meta:
        verbose_name = "Type de vente"
        verbose_name_plural = "Types de vente"
        ordering = ('designation',)
